// SPDX-License-Identifier: Apache-2.0
// Copyright Authors of K9s

package vul

const (
	// Sev1 tracks Critical sev.
	Sev1 = "SEV-1"

	// Sev2 tracks High sev.
	Sev2 = "SEV-2"

	// Sev3 tracks Medium sev.
	Sev3 = "SEV-3"

	// Sev4 tracks Low sev.
	Sev4 = "SEV-4"

	// Sev5 tracks Negligible sev.
	Sev5 = "SEV-5"

	// SevU tracks Unknown sev.
	SevU = "SEV-U"
)
